{-           						          -*-haskell-*-
GPCHaskell.chs

GPC-Haskell - A Haskell port for the GPC library.
Copyright (C) 2005  Marco Tulio Gontijo e Silva

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-}

module GPCHaskell where

import C2HS

import Foreign.Marshal.Array

data GPCVertex = GPCVertexC Double Double

type GPCCVertex = {#type gpc_nova_vertex#}

gpcVertexAlCVertex :: GPCVertex -> GPCCVertex
gpcVertexAlCVertex (GPCVertexC x y)
    = {#call fun gpc_faru_nova_vertex#} (cFloatConv x) (cFloatConv y)


gpcCVertexAlVertex :: GPCCVertex -> GPCVertex
gpcCVertexAlVertex c
    = GPCVertexC (cFloatConv $ {#call fun gpc_prenu_x#} c)
      (cFloatConv $ {#call fun gpc_prenu_y#} c)

data GPCVertexList = GPCVertexListC [GPCVertex]

type GPCCVertexList = {#type gpc_nova_vertex_list#}

gpcVertexListAlCVertexList :: GPCVertexList -> IO GPCCVertexList
gpcVertexListAlCVertexList (GPCVertexListC hlh)
    = do
      let hlc = map gpcVertexAlCVertex hlh
      hac <- newArray hlc
      return $ {#call fun gpc_faru_nova_vertex_list#} (cIntConv $ length hlh)
             hac

gpcCVertexListAlVertexList :: GPCCVertexList -> IO GPCVertexList
gpcCVertexListAlVertexList c
    = do
      let h = cIntConv $ {#call fun gpc_prenu_num_vertices#} c
          hac = {#call fun gpc_prenu_vertex#} c
      hlc <- peekArray h hac
      let hlh = map gpcCVertexAlVertex hlc
      return $ GPCVertexListC hlh

data GPCPolygon = GPCPolygonC [Int] [GPCVertexList]

type GPCCPolygon = {#type gpc_nova_polygon#}

gpcPolygonAlCPolygon :: GPCPolygon -> IO GPCCPolygon
gpcPolygonAlCPolygon (GPCPolygonC hl hlh)
    = do
      ha <- newArray $ map cIntConv hl
      hlc <- mapM gpcVertexListAlCVertexList hlh
      hac <- newArray hlc
      return $ {#call fun gpc_faru_nova_polygon#}
                 (cIntConv $ length hlh) ha hac

gpcCPolygonAlPolygon :: GPCCPolygon -> IO GPCPolygon
gpcCPolygonAlPolygon c
    = do
      let h = cIntConv $ {#call fun gpc_prenu_num_contours#} c
          hac1 = {#call fun gpc_prenu_hole#} c
          hac2 = {#call fun gpc_prenu_contour#} c
      hlc1 <- peekArray h hac1
      hlc2 <- peekArray h hac2
      let hlh1 = map cIntConv hlc1
      hlh2 <- mapM gpcCVertexListAlVertexList hlc2
      return $ GPCPolygonC hlh1 hlh2

gpcCUnion :: GPCCPolygon -> GPCCPolygon -> GPCCPolygon
gpcCUnion = {#call fun gpc_union#}

gpcUnion :: GPCPolygon -> GPCPolygon -> IO GPCPolygon
gpcUnion h1 h2
    = do
      c1 <- gpcPolygonAlCPolygon h1
      c2 <- gpcPolygonAlCPolygon h2
      gpcCPolygonAlPolygon $ gpcCUnion c1 c2
